/*
 * ServletTestHelper.java
 * 
 * Version:
 *  $Id$
 *
 * Revision: $Revision$ 
 *
 * Log: $Log$
*/
package com.actual_systems.test.client;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.net.URL;
import java.net.URLConnection;
import java.util.ArrayList;
import java.util.List;

/**
 * Helper for servlet tests
  *
  * @author terry
  * @author robert
  * @version $Revision: 1.2 $
  */
 public class ServletTestHelper {
    /**
     * send a HTTP POST message to the given URL with the given message.
     *
     * @param urlString
     * @param postData
     * @return
     * @throws IOException
     */
    public static String sendPOSTAndGetResponse(final String urlString, final String postData) throws IOException {
        InputStream is = null;
        BufferedReader br = null;

        final StringBuilder sb = new StringBuilder();
        try {

            // Create the URL object
            URL url = new URL(urlString);
            URLConnection urlConnection = url.openConnection();
            urlConnection.setDoOutput((postData != null));
            urlConnection.setDoInput(true);

            if (postData != null) {
                OutputStream outstream = urlConnection.getOutputStream();
                PrintWriter pw = new PrintWriter(outstream);
                pw.print(postData);
                pw.flush();
            }

            is = urlConnection.getInputStream();
            br = new BufferedReader(new InputStreamReader(is));

            // create a CharBuffer in order to read the Servlet Response
            //CharBuffer cb = CharBuffer.allocate(1024 * 1024);
            char[] buffer = new char[4096];
            int byteCount = br.read(buffer);

            // while we have lines to read then display them!
            while (byteCount > 0) {
                //if (logger.isDebugEnabled()) logger.debug("inputString = " + inputString);
                sb.append(buffer, 0, byteCount);

                // now read the next chunk
                byteCount = br.read(buffer);
            }

        } finally {

            // close all the connections and readers and writers
            if (is != null) {
                is.close();
            }

            if (br != null) {
                br.close();
            }
        }

        return sb.toString();
    }

    /**
     * Call the VehiclePartInputServlet to process the XML document.
     *
     * @param urlString URL to use to connect to servlet
     * @param xml XML document to call servlet with
     *
     * @return servlet responses (one entry per line)
     * @throws java.io.IOException if communication with the servlet fails
     */
    public static List<String> callServlet(final String urlString, final String xml) throws IOException {

        final String servletResponse = sendPOSTAndGetResponse(urlString, xml);
        final String[] lines = servletResponse.split("\n");
        final List<String> results = new ArrayList<String>();
        for (String line : lines) {
            results.add(line.trim());
        }
        return results;
    }

}
